@@ -8,6 +8,7 @@ from tornado.queues import Queue
 from tornado.locks import Event
 
 from IPython.core.getipython import get_ipython
+from IPython.core.inputtransformer2 import leading_empty_lines
 
 try:
     from jupyter_client.jsonutil import json_default
@@ -286,6 +287,7 @@ class Debugger:
         self.stopped_threads = []
 
         self.debugpy_initialized = False
+        self._removed_cleanup = {}
 
         self.debugpy_host = '127.0.0.1'
         self.debugpy_port = 0
@@ -341,17 +343,30 @@ class Debugger:
 
             ident, msg = self.session.recv(self.shell_socket, mode=0)
             self.debugpy_initialized = msg['content']['status'] == 'ok'
+
+        # Don't remove leading empty lines when debugging so the breakpoints are correctly positioned
+        cleanup_transforms = get_ipython().input_transformer_manager.cleanup_transforms
+        if leading_empty_lines in cleanup_transforms:
+            index = cleanup_transforms.index(leading_empty_lines)
+            self._removed_cleanup[index] = cleanup_transforms.pop(index)
+
         self.debugpy_client.connect_tcp_socket()
         return self.debugpy_initialized
 
     def stop(self):
         self.debugpy_client.disconnect_tcp_socket()
 
+        # Restore remove cleanup transformers
+        cleanup_transforms = get_ipython().input_transformer_manager.cleanup_transforms
+        for index in sorted(self._removed_cleanup):
+            func = self._removed_cleanup.pop(index)
+            cleanup_transforms.insert(index, func)
+
     async def dumpCell(self, message):
         code = message['arguments']['code']
         file_name = get_file_name(code)
 
-        with open(file_name, 'w') as f:
+        with open(file_name, 'w', encoding='utf-8') as f:
             f.write(code)
 
         reply = {
@@ -378,7 +393,7 @@ class Debugger:
         }
         source_path = message["arguments"]["source"]["path"]
         if os.path.isfile(source_path):
-            with open(source_path) as f:
+            with open(source_path, encoding='utf-8') as f:
                 reply['success'] = True
                 reply['body'] = {
                     'content': f.read()
@@ -462,6 +477,11 @@ class Debugger:
             'port': port
         }
         message['arguments']['logToFile'] = True
+        # Reverts that option for now since it leads to spurious break of the code
+        # in ipykernel source and resuming the execution leads to several errors
+        # in the kernel.
+        # Set debugOptions for breakpoints in python standard library source.
+        # message['arguments']['debugOptions'] = [ 'DebugStdLib' ]
         return await self._forward_message(message)
 
     async def configurationDone(self, message):
@@ -490,11 +510,12 @@ class Debugger:
                 'isStarted': self.is_started,
                 'hashMethod': 'Murmur2',
                 'hashSeed': get_tmp_hash_seed(),
-                'tmpFilePrefix': get_tmp_directory() + '/',
+                'tmpFilePrefix': get_tmp_directory() + os.sep,
                 'tmpFileSuffix': '.py',
                 'breakpoints': breakpoint_list,
                 'stoppedThreads': self.stopped_threads,
-                'richRendering': True
+                'richRendering': True,
+                'exceptionPaths': ['Python Exceptions']
             }
         }
         return reply
@@ -511,66 +532,53 @@ class Debugger:
 
     async def richInspectVariables(self, message):
         reply = {
-            'type': 'response',
-            'sequence_seq': message['seq'],
-            'success': False,
-            'command': message['command']
+            "type": "response",
+            "sequence_seq": message["seq"],
+            "success": False,
+            "command": message["command"],
         }
-        
-        var_name = message['arguments']['variableName']
+
+        var_name = message["arguments"]["variableName"]
         valid_name = str.isidentifier(var_name)
         if not valid_name:
-            reply['body'] = {
-                'data': {},
-                'metadata': {}
-            }
-            if var_name == 'special variables' or var_name == 'function variables':
-                reply['success'] = True
+            reply["body"] = {"data": {}, "metadata": {}}
+            if var_name == "special variables" or var_name == "function variables":
+                reply["success"] = True
             return reply
 
-        var_repr_data = var_name + '_repr_data'
-        var_repr_metadata = var_name + '_repr_metadata'
-
-        if not self.breakpoint_list:
+        repr_data = {}
+        repr_metadata = {}
+        if not self.stopped_threads:
             # The code did not hit a breakpoint, we use the intepreter
             # to get the rich representation of the variable
-            var_repr_data, var_repr_metadata = get_ipython().display_formatter.format(var_name)
+            result = get_ipython().user_expressions({var_name: var_name})[var_name]
+            if result.get("status", "error") == "ok":
+                repr_data = result.get("data", {})
+                repr_metadata = result.get("metadata", {})
         else:
             # The code has stopped on a breakpoint, we use the setExpression
             # request to get the rich representation of the variable
-            lvalue = var_repr_data + ',' + var_repr_metadata
-            code = 'get_ipython().display_formatter.format(' + var_name+')'
-            frame_id = message['arguments']['frameId']
-            seq = message['seq']
-            request = {
-                'type': 'request',
-                'command': 'setExpression',
-                'seq': seq+1,
-                'arguments': {
-                    'expression': lvalue,
-                    'value': code,
-                    'frameId': frame_id
+            code = f"get_ipython().display_formatter.format({var_name})"
+            frame_id = message["arguments"]["frameId"]
+            seq = message["seq"]
+            reply = await self._forward_message(
+                {
+                    "type": "request",
+                    "command": "evaluate",
+                    "seq": seq + 1,
+                    "arguments": {"expression": code, "frameId": frame_id},
                 }
-            }
-            await self._forward_message(request)
+            )
+            if reply["success"]:
+                repr_data, repr_metadata = eval(reply["body"]["result"], {}, {})
 
-        repr_data = globals()[var_repr_data]
-        repr_metadata = globals()[var_repr_metadata]
         body = {
-            'data': {},
-            'metadata': {}
+            "data": repr_data,
+            "metadata": {k: v for k, v in repr_metadata.items() if k in repr_data},
         }
 
-        for key, value in repr_data.items():
-            body['data']['key'] = value
-            if key in repr_metadata:
-                body['metadata'][key] = repr_metadata[key]
-
-        globals().pop(var_repr_data)
-        globals().pop(var_repr_metadata)
-
-        reply['body'] = body
-        reply['success'] = True
+        reply["body"] = body
+        reply["success"] = True
         return reply
 
     async def process_request(self, message):
