@@ -111,9 +111,11 @@ def loop_qt4(kernel):
     """Start a kernel with PyQt4 event loop integration."""
 
     from IPython.lib.guisupport import get_app_qt4
+    from IPython.external.qt_for_kernel import QtGui
 
     kernel.app = get_app_qt4([" "])
-    kernel.app.setQuitOnLastWindowClosed(False)
+    if isinstance(kernel.app, QtGui.QApplication):
+        kernel.app.setQuitOnLastWindowClosed(False)
     _notify_stream_qt(kernel, kernel.shell_stream)
 
     _loop_qt(kernel.app)
@@ -122,7 +124,16 @@ def loop_qt4(kernel):
 @register_integration('qt', 'qt5')
 def loop_qt5(kernel):
     """Start a kernel with PyQt5 event loop integration."""
-    os.environ['QT_API'] = 'pyqt5'
+    if os.environ.get('QT_API', None) is None:
+        try:
+            import PyQt5
+            os.environ['QT_API'] = 'pyqt5'
+        except ImportError:
+            try:
+                import PySide2
+                os.environ['QT_API'] = 'pyside2'
+            except ImportError:
+                os.environ['QT_API'] = 'pyqt5'
     return loop_qt4(kernel)
 
 
@@ -218,7 +229,7 @@ def loop_tk(kernel):
     # file handlers are not available on Windows
     if hasattr(app, 'createfilehandler'):
         # A basic wrapper for structural similarity with the Windows version
-        class BasicAppWrapper(object):
+        class BasicAppWrapper:
             def __init__(self, app):
                 self.app = app
                 self.app.withdraw()
@@ -242,18 +253,26 @@ def loop_tk(kernel):
         app.mainloop()
 
     else:
+        import asyncio
+        import nest_asyncio
+        nest_asyncio.apply()
+
         doi = kernel.do_one_iteration
         # Tk uses milliseconds
         poll_interval = int(1000 * kernel._poll_interval)
 
-        class TimedAppWrapper(object):
+        class TimedAppWrapper:
             def __init__(self, app, func):
                 self.app = app
                 self.app.withdraw()
                 self.func = func
 
             def on_timer(self):
-                self.func()
+                loop = asyncio.get_event_loop()
+                try:
+                    loop.run_until_complete(self.func())
+                except Exception:
+                    kernel.log.exception("Error in message handler")
                 self.app.after(poll_interval, self.on_timer)
 
             def start(self):
