@@ -13,10 +13,10 @@ from ipykernel.inprocess.blocking import BlockingInProcessKernelClient
 from ipykernel.inprocess.manager import InProcessKernelManager
 from ipykernel.inprocess.ipkernel import InProcessKernel
 from ipykernel.tests.utils import assemble_output
-from IPython.testing.decorators import skipif_not_matplotlib
 from IPython.utils.io import capture_output
 
 
+
 def _init_asyncio_patch():
     """set default asyncio policy to be compatible with tornado
 
@@ -60,12 +60,12 @@ class InProcessKernelTestCase(unittest.TestCase):
         self.kc.start_channels()
         self.kc.wait_for_ready()
 
-    @skipif_not_matplotlib
     def test_pylab(self):
         """Does %pylab work in the in-process kernel?"""
+        matplotlib = pytest.importorskip('matplotlib', reason='This test requires matplotlib')
         kc = self.kc
         kc.execute('%pylab')
-        out, err = assemble_output(kc.iopub_channel)
+        out, err = assemble_output(kc.get_iopub_msg)
         self.assertIn('matplotlib', out)
 
     def test_raw_input(self):
@@ -96,9 +96,27 @@ class InProcessKernelTestCase(unittest.TestCase):
         kc = BlockingInProcessKernelClient(kernel=kernel, session=kernel.session)
         kernel.frontends.append(kc)
         kc.execute('print("bar")')
-        out, err = assemble_output(kc.iopub_channel)
+        out, err = assemble_output(kc.get_iopub_msg)
         assert out == 'bar\n'
 
+    @pytest.mark.skip(
+        reason="Currently don't capture during test as pytest does its own capturing"
+    )
+    def test_capfd(self):
+        """Does correctly capture fd"""
+        kernel = InProcessKernel()
+
+        with capture_output() as io:
+            kernel.shell.run_cell('print("foo")')
+        assert io.stdout == "foo\n"
+
+        kc = BlockingInProcessKernelClient(kernel=kernel, session=kernel.session)
+        kernel.frontends.append(kc)
+        kc.execute("import os")
+        kc.execute('os.system("echo capfd")')
+        out, err = assemble_output(kc.iopub_channel)
+        assert out == "capfd\n"
+
     def test_getpass_stream(self):
         "Tests that kernel getpass accept the stream parameter"
         kernel = InProcessKernel()
