@@ -9,15 +9,15 @@ import signal
 import sys
 
 from IPython.core import release
-from ipython_genutils.py3compat import safe_unicode
 from IPython.utils.tokenutil import token_at_cursor, line_at_cursor
-from tornado import gen
 from traitlets import Instance, Type, Any, List, Bool, observe, observe_compat
+from zmq.eventloop.zmqstream import ZMQStream
 
 from .comm import CommManager
 from .kernelbase import Kernel as KernelBase
 from .zmqshell import ZMQInteractiveShell
 from .eventloops import _use_appnope
+from .compiler import XCachingCompiler
 
 try:
     from IPython.core.interactiveshell import _asyncio_runner
@@ -25,11 +25,21 @@ except ImportError:
     _asyncio_runner = None
 
 try:
-    from IPython.core.completer import rectify_completions as _rectify_completions, provisionalcompleter as _provisionalcompleter
+    from IPython.core.completer import (
+        rectify_completions as _rectify_completions,
+        provisionalcompleter as _provisionalcompleter,
+    )
     _use_experimental_60_completion = True
 except ImportError:
     _use_experimental_60_completion = False
 
+try:
+    import debugpy
+    from .debugger import Debugger
+    _is_debugpy_available = True
+except ImportError:
+    _is_debugpy_available = False
+
 _EXPERIMENTAL_KEY_NAME = '_jupyter_types_experimental'
 
 
@@ -42,6 +52,8 @@ class IPythonKernel(KernelBase):
         help="Set this flag to False to deactivate the use of experimental IPython completion APIs.",
     ).tag(config=True)
 
+    debugpy_stream = Instance(ZMQStream, allow_none=True) if _is_debugpy_available else None
+
     user_module = Any()
     @observe('user_module')
     @observe_compat
@@ -63,7 +75,15 @@ class IPythonKernel(KernelBase):
     _sys_eval_input = Any()
 
     def __init__(self, **kwargs):
-        super(IPythonKernel, self).__init__(**kwargs)
+        super().__init__(**kwargs)
+
+        # Initialize the Debugger
+        if _is_debugpy_available:
+            self.debugger = Debugger(self.log,
+                                    self.debugpy_stream,
+                                    self._publish_debug_event,
+                                    self.debug_shell_socket,
+                                    self.session)
 
         # Initialize the InteractiveShell subclass
         self.shell = self.shell_class.instance(parent=self,
@@ -71,6 +91,7 @@ class IPythonKernel(KernelBase):
             user_module = self.user_module,
             user_ns     = self.user_ns,
             kernel      = self,
+            compiler_class = XCachingCompiler,
         )
         self.shell.displayhook.session = self.session
         self.shell.displayhook.pub_socket = self.iopub_socket
@@ -137,27 +158,39 @@ class IPythonKernel(KernelBase):
         'file_extension': '.py'
     }
 
+    def dispatch_debugpy(self, msg):
+        if _is_debugpy_available:
+            # The first frame is the socket id, we can drop it
+            frame = msg[1].bytes.decode('utf-8')
+            self.log.debug("Debugpy received: %s", frame)
+            self.debugger.tcp_client.receive_dap_frame(frame)
+
     @property
     def banner(self):
         return self.shell.banner
 
     def start(self):
         self.shell.exit_now = False
-        super(IPythonKernel, self).start()
+        if self.debugpy_stream is None:
+            self.log.warning("debugpy_stream undefined, debugging will not be enabled")
+        else:
+            self.debugpy_stream.on_recv(self.dispatch_debugpy, copy=False)
+        super().start()
 
-    def set_parent(self, ident, parent):
+    def set_parent(self, ident, parent, channel='shell'):
         """Overridden from parent to tell the display hook and output streams
         about the parent message.
         """
-        super(IPythonKernel, self).set_parent(ident, parent)
-        self.shell.set_parent(parent)
+        super().set_parent(ident, parent, channel)
+        if channel == 'shell':
+            self.shell.set_parent(parent)
 
     def init_metadata(self, parent):
         """Initialize metadata.
 
         Run at the beginning of each execution request.
         """
-        md = super(IPythonKernel, self).init_metadata(parent)
+        md = super().init_metadata(parent)
         # FIXME: remove deprecated ipyparallel-specific code
         # This is required for ipyparallel < 5.0
         md.update({
@@ -173,9 +206,12 @@ class IPythonKernel(KernelBase):
         """
         # FIXME: remove deprecated ipyparallel-specific code
         # This is required by ipyparallel < 5.0
-        metadata['status'] = reply_content['status']
-        if reply_content['status'] == 'error' and reply_content['ename'] == 'UnmetDependency':
-                metadata['dependencies_met'] = False
+        metadata["status"] = reply_content["status"]
+        if (
+            reply_content["status"] == "error"
+            and reply_content["ename"] == "UnmetDependency"
+        ):
+            metadata["dependencies_met"] = False
 
         return metadata
 
@@ -255,9 +291,8 @@ class IPythonKernel(KernelBase):
             # restore the previous sigint handler
             signal.signal(signal.SIGINT, save_sigint)
 
-    @gen.coroutine
-    def do_execute(self, code, silent, store_history=True,
-                   user_expressions=None, allow_stdin=False):
+    async def do_execute(self, code, silent, store_history=True,
+                         user_expressions=None, allow_stdin=False):
         shell = self.shell # we'll need this a lot here
 
         self._forward_input(allow_stdin)
@@ -270,8 +305,7 @@ class IPythonKernel(KernelBase):
             should_run_async = lambda cell: False
             # older IPython,
             # use blocking run_cell and wrap it in coroutine
-            @gen.coroutine
-            def run_cell(*args, **kwargs):
+            async def run_cell(*args, **kwargs):
                 return shell.run_cell(*args, **kwargs)
         try:
 
@@ -287,17 +321,27 @@ class IPythonKernel(KernelBase):
 
             if (
                 _asyncio_runner
-                and should_run_async(code, transformed_cell=transformed_cell, preprocessing_exc_tuple=preprocessing_exc_tuple)
                 and shell.loop_runner is _asyncio_runner
                 and asyncio.get_event_loop().is_running()
+                and should_run_async(
+                    code,
+                    transformed_cell=transformed_cell,
+                    preprocessing_exc_tuple=preprocessing_exc_tuple,
+                )
             ):
-                coro = run_cell(code, store_history=store_history, silent=silent)
+                coro = run_cell(
+                    code,
+                    store_history=store_history,
+                    silent=silent,
+                    transformed_cell=transformed_cell,
+                    preprocessing_exc_tuple=preprocessing_exc_tuple
+                )
                 coro_future = asyncio.ensure_future(coro)
 
                 with self._cancel_on_sigint(coro_future):
                     res = None
                     try:
-                        res = yield coro_future
+                        res = await coro_future
                     finally:
                         shell.events.trigger('post_execute')
                         if not silent:
@@ -323,7 +367,7 @@ class IPythonKernel(KernelBase):
             reply_content.update({
                 'traceback': shell._last_traceback or [],
                 'ename': str(type(err).__name__),
-                'evalue': safe_unicode(err),
+                'evalue': str(err),
             })
 
             # FIXME: deprecated piece for ipyparallel (remove in 5.0):
@@ -377,6 +421,10 @@ class IPythonKernel(KernelBase):
                 'metadata' : {},
                 'status' : 'ok'}
 
+    async def do_debug_request(self, msg):
+        if _is_debugpy_available:
+            return await self.debugger.process_request(msg)
+
     def _experimental_do_complete(self, code, cursor_pos):
         """
         Experimental completions from IPython, using Jedi.
@@ -411,21 +459,27 @@ class IPythonKernel(KernelBase):
                 'metadata': {_EXPERIMENTAL_KEY_NAME: comps},
                 'status': 'ok'}
 
-
-
-    def do_inspect(self, code, cursor_pos, detail_level=0):
+    def do_inspect(self, code, cursor_pos, detail_level=0, omit_sections=()):
         name = token_at_cursor(code, cursor_pos)
 
         reply_content = {'status' : 'ok'}
         reply_content['data'] = {}
         reply_content['metadata'] = {}
         try:
-            reply_content['data'].update(
-                self.shell.object_inspect_mime(
+            if release.version_info >= (8,):
+                # `omit_sections` keyword will be available in IPython 8, see
+                # https://github.com/ipython/ipython/pull/13343
+                bundle = self.shell.object_inspect_mime(
+                    name,
+                    detail_level=detail_level,
+                    omit_sections=omit_sections,
+                )
+            else:
+                bundle = self.shell.object_inspect_mime(
                     name,
                     detail_level=detail_level
                 )
-            )
+            reply_content['data'].update(bundle)
             if not self.shell.enable_html_pager:
                 reply_content['data'].pop('text/html')
             reply_content['found'] = True
@@ -507,16 +561,16 @@ class IPythonKernel(KernelBase):
             # invoke IPython traceback formatting
             shell.showtraceback()
             reply_content = {
-                'traceback': shell._last_traceback or [],
-                'ename': str(type(e).__name__),
-                'evalue': safe_unicode(e),
+                "traceback": shell._last_traceback or [],
+                "ename": str(type(e).__name__),
+                "evalue": str(e),
             }
             # FIXME: deprecated piece for ipyparallel (remove in 5.0):
             e_info = dict(engine_uuid=self.ident, engine_id=self.int_id, method='apply')
             reply_content['engine_info'] = e_info
 
             self.send_response(self.iopub_socket, 'error', reply_content,
-                               ident=self._topic('error'))
+                               ident=self._topic('error'), channel='shell')
             self.log.info("Exception in apply request:\n%s", '\n'.join(reply_content['traceback']))
             result_buf = []
             reply_content['status'] = 'error'
@@ -537,4 +591,4 @@ class Kernel(IPythonKernel):
         import warnings
         warnings.warn('Kernel is a deprecated alias of ipykernel.ipkernel.IPythonKernel',
                       DeprecationWarning)
-        super(Kernel, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
