@@ -16,6 +16,10 @@ import sys
 import time
 import uuid
 import warnings
+try:
+    import psutil
+except ImportError:
+    psutil = None
 
 try:
     # jupyter_client >= 5, use tz-aware now
@@ -25,7 +29,7 @@ except ImportError:
     now = datetime.now
 
 from tornado import ioloop
-from tornado.queues import Queue
+from tornado.queues import Queue, QueueEmpty
 import zmq
 from zmq.eventloop.zmqstream import ZMQStream
 
@@ -204,10 +208,10 @@ class Kernel(SingletonConfigurable):
         'apply_request',
     ]
     # add deprecated ipyparallel control messages
-    control_msg_types = msg_types + ['clear_request', 'abort_request', 'debug_request']
+    control_msg_types = msg_types + ['clear_request', 'abort_request', 'debug_request', 'usage_request']
 
     def __init__(self, **kwargs):
-        super(Kernel, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         # Build dict of handlers for message types
         self.shell_handlers = {}
         for msg_type in self.msg_types:
@@ -441,7 +445,7 @@ class Kernel(SingletonConfigurable):
         else:
             try:
                 t, dispatch, args = self.msg_queue.get_nowait()
-            except asyncio.QueueEmpty:
+            except (asyncio.QueueEmpty, QueueEmpty):
                 return None
         await dispatch(*args)
 
@@ -702,6 +706,7 @@ class Kernel(SingletonConfigurable):
         reply_content = self.do_inspect(
             content['code'], content['cursor_pos'],
             content.get('detail_level', 0),
+            set(content.get('omit_sections', [])),
         )
         if inspect.isawaitable(reply_content):
             reply_content = await reply_content
@@ -712,7 +717,7 @@ class Kernel(SingletonConfigurable):
                                 reply_content, parent, ident)
         self.log.debug("%s", msg)
 
-    def do_inspect(self, code, cursor_pos, detail_level=0):
+    def do_inspect(self, code, cursor_pos, detail_level=0, omit_sections=()):
         """Override in subclasses to allow introspection.
         """
         return {'status': 'ok', 'data': {}, 'metadata': {}, 'found': False}
@@ -859,6 +864,39 @@ class Kernel(SingletonConfigurable):
                                       parent, ident)
         self.log.debug("%s", reply_msg)
 
+    # Taken from https://github.com/jupyter-server/jupyter-resource-usage/blob/e6ec53fa69fdb6de8e878974bcff006310658408/jupyter_resource_usage/metrics.py#L16
+    def get_process_metric_value(self, process, name, attribute=None):
+        try:
+            # psutil.Process methods will either return...
+            metric_value = getattr(process, name)()
+            if attribute is not None:  # ... a named tuple
+                return getattr(metric_value, attribute)
+            else:  # ... or a number
+                return metric_value
+        # Avoid littering logs with stack traces
+        # complaining about dead processes
+        except BaseException:
+            return None
+
+    async def usage_request(self, stream, ident, parent):
+        reply_content = {}
+        if psutil is None:
+            return reply_content
+        current_process = psutil.Process()
+        all_processes = [current_process] + current_process.children(recursive=True)
+        process_metric_value = self.get_process_metric_value
+        reply_content['kernel_cpu'] = sum([process_metric_value(process, 'cpu_percent', None) for process in all_processes])
+        reply_content['kernel_memory'] = sum([process_metric_value(process, 'memory_info', 'rss') for process in all_processes])
+        cpu_percent = psutil.cpu_percent()
+        # https://psutil.readthedocs.io/en/latest/index.html?highlight=cpu#psutil.cpu_percent
+        # The first time cpu_percent is called it will return a meaningless 0.0 value which you are supposed to ignore.
+        if cpu_percent != None and cpu_percent != 0.0:
+            reply_content['host_cpu_percent'] = cpu_percent
+        reply_content['host_virtual_memory'] = dict(psutil.virtual_memory()._asdict())
+        reply_msg = self.session.send(stream, 'usage_reply', reply_content,
+                                      parent, ident)
+        self.log.debug("%s", reply_msg)
+
     async def do_debug_request(self, msg):
         raise NotImplementedError
 
