@@ -3,7 +3,7 @@
 
 try:
     import ctypes
-except:
+except ImportError:
     ctypes = None
 import os
 import platform
@@ -22,7 +22,7 @@ class ParentPollerUnix(Thread):
     """
 
     def __init__(self):
-        super(ParentPollerUnix, self).__init__()
+        super().__init__()
         self.daemon = True
 
     def run(self):
@@ -55,13 +55,12 @@ class ParentPollerWindows(Thread):
         interrupt_handle : HANDLE (int), optional
             If provided, the program will generate a Ctrl+C event when this
             handle is signaled.
-
         parent_handle : HANDLE (int), optional
             If provided, the program will terminate immediately when this
             handle is signaled.
         """
         assert(interrupt_handle or parent_handle)
-        super(ParentPollerWindows, self).__init__()
+        super().__init__()
         if ctypes is None:
             raise ImportError("ParentPollerWindows requires ctypes")
         self.daemon = True
