@@ -4,8 +4,9 @@
 # Distributed under the terms of the Modified BSD License.
 
 import ast
-import io
 import os.path
+import platform
+import subprocess
 import sys
 import time
 from tempfile import TemporaryDirectory
@@ -14,7 +15,6 @@ from flaky import flaky
 import pytest
 from packaging import version
 
-from IPython.testing import decorators as dec, tools as tt
 import IPython
 from IPython.paths import locate_profile
 
@@ -185,6 +185,7 @@ def test_subprocess_error():
         _check_master(kc, expected=True)
         _check_master(kc, expected=True, stream="stderr")
 
+
 # raw_input tests
 
 def test_raw_input():
@@ -219,14 +220,14 @@ def test_save_history():
         wait_for_idle(kc)
         _, reply = execute("%hist -f " + file, kc=kc)
         assert reply['status'] == 'ok'
-        with io.open(file, encoding='utf-8') as f:
+        with open(file, encoding='utf-8') as f:
             content = f.read()
         assert 'a=1' in content
         assert 'b="abcþ"' in content
 
 
-@dec.skip_without('faulthandler')
 def test_smoke_faulthandler():
+    faulthadler = pytest.importorskip('faulthandler', reason='this test needs faulthandler')
     with kernel() as kc:
         # Note: faulthandler.register is not available on windows.
         code = '\n'.join([
@@ -242,7 +243,12 @@ def test_smoke_faulthandler():
 
 def test_help_output():
     """ipython kernel --help-all works"""
-    tt.help_all_output_test('kernel')
+    cmd = [sys.executable, "-m", "IPython", "kernel", "--help-all"]
+    proc = subprocess.run(cmd, timeout=30, capture_output=True)
+    assert proc.returncode == 0, proc.stderr
+    assert b"Traceback" not in proc.stderr
+    assert b"Options" in proc.stdout
+    assert b"Class" in proc.stdout
 
 
 def test_is_complete():
@@ -296,8 +302,8 @@ def test_complete():
         assert completed.startswith(cell)
 
 
-@dec.skip_without('matplotlib')
 def test_matplotlib_inline_on_import():
+    pytest.importorskip('matplotlib', reason='this test requires matplotlib')
     with kernel() as kc:
         cell = '\n'.join([
             'import matplotlib, matplotlib.pyplot as plt',
@@ -344,7 +350,7 @@ def test_unc_paths():
         file_path = os.path.splitdrive(os.path.dirname(drive_file_path))[1]
         unc_file_path = os.path.join(unc_root, file_path[1:])
 
-        kc.execute("cd {0:s}".format(unc_file_path))
+        kc.execute(f"cd {unc_file_path:s}")
         reply = kc.get_shell_msg(timeout=TIMEOUT)
         assert reply['content']['status'] == 'ok'
         out, err = assemble_output(kc.get_iopub_msg)
@@ -362,6 +368,10 @@ def test_unc_paths():
         assert reply['content']['status'] == 'ok'
 
 
+@pytest.mark.skipif(
+    platform.python_implementation() == "PyPy",
+    reason="does not work on PyPy",
+)
 def test_shutdown():
     """Kernel exits after polite shutdown_request"""
     with new_kernel() as kc:
@@ -416,8 +426,8 @@ def test_interrupt_with_message():
 
 
 @pytest.mark.skipif(
-    version.parse(IPython.__version__) < version.parse("7.14.0"),
-    reason="Need new IPython"
+    "__pypy__" in sys.builtin_module_names,
+    reason="fails on pypy",
 )
 def test_interrupt_during_pdb_set_trace():
     """
