@@ -2,7 +2,6 @@
 # Distributed under the terms of the Modified BSD License.
 
 import json
-import io
 import os
 import shutil
 import sys
@@ -23,25 +22,23 @@ from ipykernel.kernelspec import (
 
 import pytest
 
-import nose.tools as nt
-
 pjoin = os.path.join
 
 
 def test_make_ipkernel_cmd():
     cmd = make_ipkernel_cmd()
-    nt.assert_equal(cmd, [
+    assert cmd == [
         sys.executable,
         '-m',
         'ipykernel_launcher',
         '-f',
         '{connection_file}'
-    ])
+    ]
 
 
 def assert_kernel_dict(d):
     assert d['argv'] == make_ipkernel_cmd()
-    assert d['display_name'] == 'Python %i' % sys.version_info[0]
+    assert d['display_name'] == 'Python %i (ipykernel)' % sys.version_info[0]
     assert d['language'] == 'python'
 
 
@@ -51,10 +48,9 @@ def test_get_kernel_dict():
 
 
 def assert_kernel_dict_with_profile(d):
-    nt.assert_equal(d['argv'], make_ipkernel_cmd(
-        extra_arguments=["--profile", "test"]))
-    assert d['display_name'] == 'Python %i' % sys.version_info[0]
-    assert d['language'] == 'python'
+    assert d["argv"] == make_ipkernel_cmd(extra_arguments=["--profile", "test"])
+    assert d["display_name"] == "Python %i (ipykernel)" % sys.version_info[0]
+    assert d["language"] == "python"
 
 
 def test_get_kernel_dict_with_profile():
@@ -68,7 +64,7 @@ def assert_is_spec(path):
         assert os.path.exists(dst)
     kernel_json = pjoin(path, 'kernel.json')
     assert os.path.exists(kernel_json)
-    with io.open(kernel_json, encoding='utf8') as f:
+    with open(kernel_json, encoding='utf8') as f:
         json.load(f)
 
 
@@ -89,10 +85,9 @@ def test_write_kernel_spec_path():
 def test_install_kernelspec():
 
     path = tempfile.mkdtemp()
-    try: 
-        test = InstallIPythonKernelSpecApp.launch_instance(argv=['--prefix', path])
-        assert_is_spec(os.path.join(
-            path, 'share', 'jupyter', 'kernels', KERNEL_NAME))
+    try:
+        InstallIPythonKernelSpecApp.launch_instance(argv=["--prefix", path])
+        assert_is_spec(os.path.join(path, "share", "jupyter", "kernels", KERNEL_NAME))
     finally:
         shutil.rmtree(path)
 
@@ -128,7 +123,7 @@ def test_install_profile():
     with open(spec) as f:
         spec = json.load(f)
     assert spec["display_name"].endswith(" [profile=Test]")
-    nt.assert_equal(spec["argv"][-2:], ["--profile", "Test"])
+    assert spec["argv"][-2:] == ["--profile", "Test"]
 
 
 def test_install_display_name_overrides_profile():
@@ -165,4 +160,3 @@ def test_install_env(tmp_path, env):
             assert spec['env'][k] == v
     else:
         assert 'env' not in spec
-
