@@ -5,20 +5,20 @@
 
 import atexit
 import os
+import platform
 import sys
+from tempfile import TemporaryDirectory
 from time import time
 
 from contextlib import contextmanager
 from queue import Empty
-from subprocess import PIPE, STDOUT
-
-import nose
+from subprocess import STDOUT
 
 from jupyter_client import manager
 
 
 STARTUP_TIMEOUT = 60
-TIMEOUT = 15
+TIMEOUT = 100
 
 KM = None
 KC = None
@@ -31,8 +31,9 @@ def start_new_kernel(**kwargs):
     """
     kwargs['stderr'] = STDOUT
     try:
+        import nose
         kwargs['stdout'] = nose.iptest_stdstreams_fileno()
-    except AttributeError:
+    except (ImportError, AttributeError):
         pass
     return manager.start_new_kernel(startup_timeout=STARTUP_TIMEOUT, **kwargs)
 
@@ -43,23 +44,25 @@ def flush_channels(kc=None):
 
     if kc is None:
         kc = KC
-    for channel in (kc.shell_channel, kc.iopub_channel):
+    for get_msg in (kc.get_shell_msg, kc.get_iopub_msg):
         while True:
             try:
-                msg = channel.get_msg(block=True, timeout=0.1)
+                msg = get_msg(timeout=0.1)
             except Empty:
                 break
             else:
                 validate_message(msg)
 
 
-def get_reply(kc, msg_id, timeout):
-    timeout = TIMEOUT
+def get_reply(kc, msg_id, timeout=TIMEOUT, channel='shell'):
     t0 = time()
     while True:
-        reply = kc.get_shell_msg(timeout=timeout)
+        get_msg = getattr(kc, f'get_{channel}_msg')
+        reply = get_msg(timeout=timeout)
         if reply['parent_header']['msg_id'] == msg_id:
             break
+        # Allow debugging ignored replies
+        print(f"Ignoring reply not to {msg_id}: {reply}")
         t1 = time()
         timeout -= t1 - t0
         t0 = t1
@@ -142,19 +145,20 @@ def new_kernel(argv=None):
     """
     kwargs = {'stderr': STDOUT}
     try:
+        import nose
         kwargs['stdout'] = nose.iptest_stdstreams_fileno()
-    except AttributeError:
+    except (ImportError, AttributeError):
         pass
     if argv is not None:
         kwargs['extra_arguments'] = argv
     return manager.run_kernel(**kwargs)
 
-def assemble_output(iopub):
+def assemble_output(get_msg):
     """assemble stdout/err from an execution"""
     stdout = ''
     stderr = ''
     while True:
-        msg = iopub.get_msg(block=True, timeout=1)
+        msg = get_msg(timeout=1)
         msg_type = msg['msg_type']
         content = msg['content']
         if msg_type == 'status' and content['execution_state'] == 'idle':
@@ -174,8 +178,27 @@ def assemble_output(iopub):
 
 def wait_for_idle(kc):
     while True:
-        msg = kc.iopub_channel.get_msg(block=True, timeout=1)
+        msg = kc.get_iopub_msg(timeout=1)
         msg_type = msg['msg_type']
         content = msg['content']
         if msg_type == 'status' and content['execution_state'] == 'idle':
             break
+
+
+class TemporaryWorkingDirectory(TemporaryDirectory):
+    """
+    Creates a temporary directory and sets the cwd to that directory.
+    Automatically reverts to previous cwd upon cleanup.
+    Usage example:
+
+        with TemporaryWorkingDirectory() as tmpdir:
+            ...
+    """
+    def __enter__(self):
+        self.old_wd = os.getcwd()
+        os.chdir(self.name)
+        return super().__enter__()
+
+    def __exit__(self, exc, value, tb):
+        os.chdir(self.old_wd)
+        return super().__exit__(exc, value, tb)
